.\"
.\" $(TROFF) $(MSMACROS) file
.\"
.\" these macros are used in addition to the -ms macros
.so tmac.wrprc
.\"
.\" revision date - change whenever this file is edited
.ds RD 6 April 1994
.nr PO 1.25i	\" page offset 1.25 inches
.nr PD .5v	\" inter-paragraph distance
.\"
.EH 'RTF Macintosh Driver Architecture'- % -''
.OH ''- % -'RTF Macintosh Driver Architecture'
.OF 'Revision date:\0\0\*(RD''Printed:\0\0\n(dy \*(MO 19\n(yr'
.EF 'Revision date:\0\0\*(RD''Printed:\0\0\n(dy \*(MO 19\n(yr'
.\"
.TL
.ps +2
RTF Tools
.sp .3v
Macintosh Driver Architecture
.AU
Paul DuBois
dubois@primate.wisc.edu
.AI
Wisconsin Regional Primate Research Center
Revision date:\0\0\*(RD
.\"
.Ah Introduction
.\"
.LP
This document accompanies
.I "RTF Tools Translator Architecture" ,
which you are assumed to be familiar with.
It describes how the Macintosh translator drivers are set up.
.\"
.Ah "The AppInfo Structure"
.\"
.LP
The Macintosh versions of the RTF translators included in the RTF Tools
distribution provide a standard Macintosh interface (windows, menus, etc.).
They all use pretty much the same driver code, so there is a reasonable
amount of consistency from translator to translator.
In fact, most of the driver is absorbed into a wrapper that's provided as
a library.
When a translator starts up, the driver calls the wrapper and passes it
a pointer to an
.I AppInfo
structure that contains information instructing the wrapper how to behave:
.Ps
int
main (void)
{
	MacintoshWrapper (&myAppInfo);
}
.Pe
The structure looks like this:
.Ps
typedef	struct AppInfo	AppInfo;

struct AppInfo
{
	void	(*pMenuInit) ();		/* application-specific menu setup */
	void	(*pMenuAdjust) ();		/* application-specific menu adjustment */

	void	(*pWriterInit) ();		/* writer one-time initialization */
	void	(*pWriterEnd) ();		/* writer one-time cleanup */

	int		(*pWriterBeginFile) ();	/* per-file initialization */
	void	(*pWriterEndFile) ();	/* per-file cleanup */

	void	(*pPrefs) ();			/* function to handle Preferences dialog */

	Str31	inputSuffix;			/* input file suffix to strip if present */
	Str31	outputSuffix;			/* output file suffix for autonaming */
	Str31	errorSuffix;			/* error file suffix for autonaming */

	long	outputCreator;			/* creator of output files */
	long	errorCreator;			/* creator of error files */

	int		flags;					/* flags to control wrapper behavior */
};
.Pe
This structure contains several function pointers.
If the translator has no need for a given function, specify
.Cw nil
in the
.I AppInfo
structure.
.LP
The structure members are used as follows:
.\"
.IP \f(CWpMenuInit\fP
.sp .5v
A pointer to a function to call that allows the driver to add its own
menus to the menu bar.
It's defined like this:
.Ps
void
MyMenuInit (void)
{
}
.Pe
The wrapper provides Apple, File, and Edit menus.
If the driver needs to add other menus, it should do so in this function.
The function doesn't need to call
.I DrawMenuBar()
because the wrapper calls it after calling the function.
.\"
.IP \f(CWpMenuAdjust\fP
.sp .5v
A pointer to a function to call when a mouse click occurs in the menu bar.
It's defined like this:
.Ps
void
MyMenuAdjust (void)
{
}
.Pe
If the driver provides any of its own menus and the state of any items in them
need to be adjusted (enabled, disabled, checked, etc.) before the user sees
them, use this function to do so.
.\"
.IP \f(CWpWriterInit\fP
.sp .5v
A pointer to a function that performs one-time writer initialization
before any input files are processed.
It's defined like this:
.Ps
void
MyWriterInit (void)
{
}
.Pe
.\"
.IP \f(CWpWriterEnd\fP
.sp .5v
A pointer to a function that performs one-time writer cleanup after all
input files have been processed.
It's defined like this:
.Ps
void
MyWriterEnd (void)
{
}
.Pe
.\"
.IP \f(CWpWriterBeginFile\fP
.sp .5v
A pointer to a function to call to tell the writer to get ready to generate
a new output file.
It's defined like this:
.Ps
int
MyWriterBeginFile (void)
{
}
.Pe
When this function is called, the wrapper will already have opened the input
file and called
.I RTFInit() ,
installed the input file name into the RTF library by calling
.I RTFSetInputName() ,
and opened the error file if there is one.
If there is an output file, the wrapper will have installed its name by
calling
.I RTFSetOutputName() ,
and, if the
.Cw autoOpenOutputFile
flag is set (see below), will have opened it on
.I stdout .
.sp .5v
If the wrapper doesn't open the output file and one is to be generated,
this function must open it.
.sp .5v
The function should return non-zero for success (it was able to set up
for a new file), zero otherwise (setup failed).
.\"
.IP \f(CWpWriterEndFile\fP
.sp .5v
A pointer to a function to call to tell the writer to clean up after an
input file has been read.
It's defined like this:
.Ps
void
MyWriterEndFile (void)
{
}
.Pe
The function should generate any final output that may be necessary.
If the wrapper opened the output file, the wrapper will also close it
after this function returns.
Otherwise the function should close the file itself.
.\"
.IP \f(CWpPrefs\fP
.sp .5v
A pointer to a function to call to process a preferences dialog.
It's defined like this:
.Ps
void
MyPrefsDialog (void)
{
}
.Pe
This function is ignored by the wrapper unless the
.Cw prefsInEditMenu
flags is set (see below).
If the flag is set, the wrapper adds a gray line and a ``Preferences...''
item to the bottom of the Edit menu and calls this function when the user
selects the item.
.\"
.IP \f(CWinputSuffix\fP
.br
.IP \f(CWoutputSuffix\fP
.br
.IP \f(CWerrorSuffix\fP
.sp .5v
These suffixes are used in constructing default output and error file names.
They should be specified as Pascal strings and must be
.Cw nil . non-
.sp .5v
The names are constructed by stripping the input suffix off the input file
name if it's present and then adding the output or error suffix.
The input and error suffixes are typically
.Cw .rtf
and
.Cw .err .
The output suffix varies by translator.
.sp .5v
.B Example:
If the input, output, and error suffixes are
.Cw .rtf ,
.Cw .txt ,
and
.Cw .err ,
the default names for the input file
.I testfile.rtf
will be
.I testfile.txt
and
.I testfile.err .
Since the input suffix is only stripped if it's present, the default
output and error file names for an input file named
.I testfile
would be the same.
If the input file were
.I testfile.doc ,
the default names would be
.I testfile.doc.txt
and
.I testfile.doc.err .
.sp .5v
The output and error suffixes are not used if the translator doesn't
generate output and error files, but if it does, the suffixes should not
be empty.
.\"
.IP \f(CWoutputCreator\fP
.br
.IP \f(CWerrorCreator\fP
.sp .5v
Creator signatures of the applications that should own output and error files
created by the translator.
A default of
.Cw 'ttxt'
(TeachText) is used if the creator is given as zero in the
.I AppInfo
structure.
.sp .5v
Output and error files, if created by the wrapper, are always
.Cw 'TEXT'
files.
If the translator wants to create output files with a type other than
.Cw 'TEXT' ,
it should tell the wrapper not to open output files and should open
them itself.
However,
since file type is a Macintosh-specific thing and since the translator's
writer code should be written to be system-independent, the writer cannot
portably be written to set the file type.
Once way to deal with this problem is to have the driver provide a function
for opening output files which is called by the
.I pWriterBeginFile
function and which is appropriate for the type of system for which the
driver is written.
.\"
.IP \f(CWflags\fP
.sp .5v
Flags controlling how the wrapper behaves.
The value of this member should be zero or constructed as a combination of
the following flags, or'ed together:
.in +.5i
.IP \f(CWpersistentStatus\fP
.sp .5v
If
.Cw persistentStatus
is set, the status window is always visible.
Otherwise, it's visible only during file translations.
.IP \f(CWprefsInEditMenu\fP
.sp .5v
If
.Cw prefsInEditMenu
is set, the wrapper adds a gray line and a
``Preferences...'' item to the bottom of the Edit menu, and calls the
.Cw pPrefs
function in the
.I AppInfo
structure to process the preferences dialog when the user selects
the item.
.IP \f(CWusesOutputFile\fP
.sp .5v
If
.Cw usesOutputFile
is set, it means the translator expects to write to
an output file.
If the flag is set, the
.Cw autoNameOutputFile
and
.Cw autoOpenOutputFile
flags are also examined (they're ignored if
.Cw usesOutputFile
isn't set).
.IP \f(CWautoNameOutputFile\fP
.sp .5v
If
.Cw autoNameOutputFile
is set, the output filename is automatically
generated, and the user can't override it.
If the flag isn't set, the name is still generated and used as the
suggested default.
The user can override it by means of a standard file dialog.
.IP \f(CWautoOpenOutputFile\fP
.sp .5v
If
.Cw autoOpenOutputFile
is set, the output file is opened by the wrapper on
.I stdout.
Otherwise, the writer is expected to open the output file itself.
.IP \f(CWusesErrorFile\fP
.br
.IP \f(CWautoNameErrorFile\fP
.sp .5v
.Cw usesErrorFile
and
.Cw autoNameErrorFile
are similar to
.Cw autoNameOutputFile
and
.Cw autoOpenOutputFile ,
but they apply to the error file.
There is no
.Cw autoOpenErrorFile ;
if an error file name is used, the wrapper always
opens it (on
.I stderr ).
.sp .5v
The writer can call
.I RTFMsg()
and
.I RTFPanic()
to generate diagnostic output even if no error file is created,
because such output goes to the log window anyway.
.in -.5i
.\"
.Ah "Subverting pWriterInit"
.\"
.LP
If the driver needs to do its own initialization (e.g., to read a preferences
file), it can do this by specifying a driver
function instead of a writer function for the
.I pWriterInit
member of the
.I AppInfo
structure.
You can use this function to initialize the driver and then call
any writer initialization function that may be necessary.
When the driver initialization function is called, the
.I pMenuInit
function will already have been called, but none of the rest of the
.I AppInfo
structure will have been used.
This means you can read a preferences file to get suffix settings and so forth
and install them in the
.I AppInfo
structure with the expectation that the wrapper will use them properly.
