.\"
.\" $(TROFF) $(MSMACROS) file
.\"
.\" these macros are used in addition to the -ms macros
.so tmac.wrprc
.\"
.\" revision date - change whenever this file is edited
.ds RD 5 April 1994
.nr PO 1.25i	\" page offset 1.25 inches
.nr PD .5v	\" inter-paragraph distance
.\"
.EH 'RTF Translator Architecture'- % -''
.OH ''- % -'RTF Translator Architecture'
.OF 'Revision date:\0\0\*(RD''Printed:\0\0\n(dy \*(MO 19\n(yr'
.EF 'Revision date:\0\0\*(RD''Printed:\0\0\n(dy \*(MO 19\n(yr'
.\"
.TL
.ps +2
RTF Tools
.sp .3v
Translator Architecture
.AU
Paul DuBois
dubois@primate.wisc.edu
.AI
Wisconsin Regional Primate Research Center
Revision date:\0\0\*(RD
.\"
.Ah Introduction
.\"
.LP
The RTF translators included in the RTF Tools distribution were revised
for release 1.10 to provide a more flexible architecture that
makes it easier to port the writer end of a translator to different systems.
This document describes some of the problems the architecture is intended
to solve and how it tries to solve them.
A companion document,
.I "RTF Tools Macintosh Driver Architecture" ,
contains notes specific to the Macintosh translator drivers.
.\"
.Ah "The Original Translator Architecture"
.\"
.LP
The translators in the RTF Tools distribution have always used a
design that factors the reader and writer code into separate entities.
The reader and writer are linked together by a driver which sets up the
input and output files, installs writer functions into the reader to be
called when tokens of particular classes occur, and then
calls the reader to process the input stream and pass tokens to the
appropriate writer routines automatically.
.LP
Here is a simple example of a driver for a translator
that can read an input file
from the standard input or from a named file on a UNIX system:
.Ps
int
main (argc, argv)
int		argc;
char	*argv[];
{
	if (argc > 1)	/* if a file was named, open it */
	{
		if (freopen (argv[1], "r", stdin) == (FILE *) NULL)
		{
			fprintf (stderr, "cannot open %s\en", argv[1]);
			exit (1);	/* terminate abnormally */
		}
		fprintf (stderr, "processing %s...\en", argv[1]);
	}

	/*
	 * Initialize the reader, install a callback to catch text
	 * tokens, and read the input stream.
	 */

	RTFInit ();
	RTFSetClassCallback (rtfText, TextClass);
	RTFRead ();

	exit (0);	/* terminate normally */
}
.Pe
This driver installs a callback
.I TextClass()
into the reader
to be called when text tokens occur in the input stream.
.I TextClass()
could be written like this to make the
translator a simple program for extracting text from RTF files:
.Ps
void
TextClass ()
{
	putchar (rtfMajor);	/* character is in token major number */
}
.Pe
Note that
.I TextClass()
is the entire text of the writer code for the translator.
Tokens other than text tokens will be ignored.
Most translators ramify the writer code considerably beyond this
minimal effort.
.\"
.Ah "Limitations of the Original Architecture"
.\"
.LP
The translator just described reflects one of the primary
characteristics exhibited by
most of the translators included in the RTF Tools distribution as they
were originally written.
That is, they were built as single-file
converters conceived for command-line use in a UNIX environment.
They read a file and wrote the translated output to the
standard output.
.LP
Such a translation model has limited applicability to other environments.
For instance, under System 7 on a Macintosh
it's desirable to take advantage of the drag-and-drop facility to allow the
user to drop an arbitrary number of documents onto the translator with the
expectation that each of them will be translated in sequence.
For that to work, it must be possible to ``restart'' the translator
for each file to be processed, without carrying over from file to file
any file-specific state.
Unfortunately, although the RTF reader has always been written to be
restartable, the writer code for many of the translators has not been.
.LP
The example also demonstrates other system dependencies that have characterized
the RTF Tools translators:
.Ls B
.Li
Diagnostic messages are written to
.I stderr .
.Li
When a fatal error occurs, the process is terminated by calling
.I exit()
after writing a message to
.I stderr .
.Le
These actions are appropriate for UNIX, but may not be for other environments.
For instance, under System 7 it may be
preferable to write diagnostic messages to a window rather than to
.I stderr .
But if you exit immediately after writing a message to a window
when a fatal error occurs, the message disappears.
This is not helpful to the user.
It might be preferable to present the message in an alert that persists
until the user dismisses it.
.LP
Another OS-dependent operation that's not shown in the example translator
is the mechanism for locating library files such as the charset maps.
Under UNIX these files are assumed to be located in a well-known directory
such as
.I /usr/local/lib/rtf-tools .
Under System 7 they might be best located inside the
.I Preferences
folder, or perhaps in the same folder as that in which the application itself
is located.
.LP
The translators included with the RTF Tools distribution
were revised for release 1.10 to alleviate these problems.
Specifically:
.Ls B
.Li
Reader and writer code is more system-independent.
.Li
Writers are restartable so they can process multiple files.
.Le
.\"
.Ah "Handling Operating System Dependencies"
.\"
.LP
In order to accommodate system dependencies, a given translator is compiled
on different systems with the same reader and writer code, but a separate
driver is used for each system on which the translator is built.
When it starts up, the driver provides callback
functions for the OS-specific operations and plugs them into the RTF library:
.Ps
RTFSetMsgProc (Msg);
RTFSetPanicProc (Panic);
RTFSetOpenLibFileProc (OpenLibFile);
.Pe
Thereafter, the translator calls generic routines that map onto the OS-specific
functions.
For example:
.Ps
/* display message */
RTFMsg ("processing input file %s\en", inputName);

/* abort */
RTFPanic ("cannot open %s", outputName);

/* open library file */
f = RTFOpenLibFile ("ansi-gen", "r);
.Pe
This mechanism localizes the system dependencies into the driver,
allowing the reader and writer to refer to standard function
names without regard to the type of system on which the program
is running.
.\"
.Bh "Diagnostic Output"
.\"
.LP
To handle diagnostic output, the driver should
provide a function that takes a single
null-terminated string argument and displays it to the user:
.Ps
void
Msg (s)
char	*s;
{
	/* print s */
}
.Pe
When the translator starts up, install the routine like this:
.Ps
RTFSetMsgProc (Msg);
.Pe
To write a diagnostic message from your translator, call
.I RTFMsg() .
.I RTFMsg()
takes
.I printf() -like
arguments, but it formats the message into a single string before passing
it to your message function so your function doesn't have to understand
a variable number of arguments.
Newlines in the string indicate linebreaks; your callback
may need to convert them to something else if newlines are inappropriate
linebreak characters for your system.
.LP
The default message function writes the message to
.I stderr .
If this is suitable, your translator doesn't need to provide a message
function.
.\"
.Bh "Process Termination"
.\"
.LP
To terminate the translator when a fatal error occurs,
the driver should
provide a function that takes a single
null-terminated string argument, displays it to the user, and exits:
.Ps
void
Panic (s)
char	*s;
{
	/* print s */
	/* terminate process */
}
.Pe
Your panic function
.I must
terminate the process, in whatever way is
appropriate for your system. (e.g., by calling
.I exit()
under UNIX or by
calling
.I ExitToShell()
on a Macintosh).
.LP
When the translator starts up, install the routine like this:
.Ps
RTFSetPanicProc (Panic);
.Pe
To terminate your translator, call
.I RTFPanic() .
Like
.I RTFMsg() ,
.I RTFPanic()
takes
.I printf() -like
arguments and formats them into a single string which it passes to the
OS-specific routine.
Unlike
.I RTFMsg() ,
.I RTFPanic()
adds a newline to the end of the message and indicates what token was
last read from the input stream.
Newlines in the string indicate linebreaks; your callback
may need to convert them to something else.
.LP
.I RTFPanic()
doesn't pass your panic function any exit status; your function
can supply one if that's desired.
.LP
The default panic function writes the message to
.I stderr
and calls
.I exit()
with a status of 1.
If this is suitable, your translator doesn't need to provide a panic function.
.\"
.Bh "Library File Access"
.\"
.LP
For opening library files,
the driver should
provide an OS-specific routine that takes a basename
and access mode, locates the file and opens it, and returns a
.Cw FILE
pointer:
.Ps
FILE *
OpenLibFile (name, mode)
char	*name;
char	*mode;
{
	/* find and open the file */
	/* return FILE pointer, or NULL if the file cannot be opened */
}
.Pe
When the translator starts up, install the routine like this:
.Ps
RTFSetOpenLibFileProc (OpenLibFile);
.Pe
The translator opens library files by calling
.I RTFOpenLibFile().
Library files should be specified by basename rather than by full
pathname because pathnames vary in form among systems.
.LP
There is no default library file opening function.
If your translator doesn't provide one,
.I RTFOpenLibFile()
always returns
.Cw NULL .
.\"
.Ah "The Driver/Writer Interface"
.\"
.LP
In addition to providing OS-specific callbacks so the reader and
writer can perform OS-dependent tasks in an OS-independent way,
the driver must also
translate information from its operating environment
into a form the reader and writer can use:
.Ls B
.Li
The driver handles the
details of the file-delivery mechanism provided by the OS
to determine which files are to be translated.
For UNIX, these are named on the command line and are available in
.I argv[] .
For System 7, they're delivered through Apple Events.
.Li
For each input file, the driver takes care of opening it,
initializing the reader, and telling the writer when to begin and
end output file processing.
.Li
The driver transmits user preference information to the writer.
For UNIX, these are named on the command line and are available in
.I argv[] .
For System 7, they can be specified in preferences files or via dialogs.
.Le
All of these are issues involving the interface between the driver and the
writer.
The next few sections describe the interface used by the translators in the
RTF Tools distribution.
Note that it's one I find convenient; it's not the only interface
that's possible.
.\"
.Bh "Structuring the Writer"
.\"
.LP
This section describes how the writer code for a translator
can be structured to allow for multiple file processing.
As described below, there are four primary writer interface calls,
although they need not all be applicable to any given translator.
When present, the functions are written as shown below.
None of them take any arguments and none except
.I WriterBeginFile()
return a value.
.Ps
void
WriterInit ()
{
	/* perform any once-only initializations */
}
.Pe
.Ps
void
WriterEnd ()
{
	/* perform any once-only cleanup */
}
.Pe
.Ps
int
WriterBeginFile ()
{
	/* prepare to generate new output file */
	/* return value indicating success or failure of setup */
}
.Pe
.Ps
void
WriterEndFile()
{
	/* write any final per-file output */
}
.Pe
In general, a driver uses the writer
functions in a sequence something like this:
.Ps
WriterInit ();	/* do once-only writer initialization */

for (/* each file to be translated */)
{
	freopen (..., stdin);	/* open input and output files */
	freopen (..., stdout);
	RTFInit();				/* initialize reader */
	if (WriterBeginFile ())	/* do per-file writer initialization */
	{
		RTFRead ();			/* read the input file */
		WriterEndFile ();	/* do per-file writer cleanup */
	}
}

WriterEnd ();	/* do once-only writer cleanup */
.Pe
.\"
.Ch "Writer Initialization and Termination"
.\"
.LP
If your writer must perform one-time initialization
before any input files are processed, or one-time cleanup
after all files have been processed, provide the functions
.I WriterInit()
and
.I WriterEnd() .
.LP
.I WriterInit()
is called once before any files are to be translated,
after the OS-specific callbacks have been installed.
This allows the writer to perform any one-time initializations.
One thing it might do is
read the output map if there is one, since this generally need be done only
once.
.I rtf2troff "" (
reads output maps on a per-input file basis, but that is
the exception that proves the rule).
.LP
.I WriterEnd()
is called after all the input files have been processed.
.\"
.Ch "Processing the Input Files"
.\"
.LP
Between the calls to
.I WriterInit()
and
.I WriterEnd() ,
the translator processes the input files.
For each one the driver should do the following:
.Ls B
.Li
Open the input file on
.I stdin
and call
.I RTFInit() .
.Li
Open the output file if the writer doesn't do so.
(Either the driver or the writer may open/close the output
file, but of course they must agree between themselves which of them will
do it.)
.Li
If the writer needs to be called to begin each output file, call
.I WriterBeginFile() .
The responsibilities of this function will be described shortly.
.Li
After calling
.I WriterBeginFile() ,
call the RTF library to
read the input file.
This can be done two ways.
You can call
.I RTFRead()
to read and process all the tokens in the input stream in one fell swoop.
Alternatively, you can read and process them individually with a loop:
.Ps
while (RTFGetToken () != rtfEOF)	/* read and process one */
{									/* token at a time */
	RTFRouteToken ();
}
.Pe
.I RTFRead()
is in fact a function consisting of the loop just shown.
The reason you might want to use an explicit loop in the driver
instead of calling
.I RTFRead()
is that you get control back each time through the loop.
This allows you to take other actions in addition to calling
.I RTFRouteToken() ,
which may be important on some systems.
.sp .5v
.I RTFRead()
is sufficient for a system like UNIX where you can forcibly terminate
a running process by typing the interrupt character.
It's a considerably different undertaking
to do the translation in a Macintosh event-loop user-driven
environment where no external means is provided for terminating processes.
The process itself must detect when the user wants an early
quit, so
it's desirable to make a translator
responsive to the user during file processing, rather than just between
files.
.sp .5v
.I RTFRead()
doesn't work well under such circumstances.
If it were to
be called to process the input stream, control would disappear into the
reader and writer code (which, being system independent,
knows nothing of the Macintosh user interface
code), until the file was entirely processed.
This is bad because it locks out the user during the translation.
To get around this, a normal
event loop is run, and individual tokens are read from the input stream and
processed when null events occur.
This allows the user to select menu items (e.g., to request a quit, or to
add new documents to the translation queue) while a translation is in
progress.
It also allows a progress bar and spinning cursor to be
displayed.
.Li
If the writer requires any per-file cleanup
(e.g., to write any final output to the output stream)
after the input stream has been exhausted,
call
.I WriterEndFile() .
.Li
Close the input file.
.Le
.LP
.B "Duties of WriterBeginFile()"
.LP
In brief,
.I WriterBeginFile()
sets up the writer to begin producing a new output file and returns
a status value to the driver.
.LP
.I WriterBeginFile()
can assume that the driver has already opened the input file and called
.I RTFInit() .
.LP
.I WriterBeginFile()
sets the writer's state to what it should be at the beginning of each file.
No state specific to the previous file should be retained.
Typically this initialization involves the following:
.Ls B
.Li
If the driver opens the file, it does so on
.I stdout .
If the driver doesn't open the output file, the writer should do so.
.sp .5v
If
.I WriterBeginFile()
needs to know the names of the input or output files,
they may be obtained by calling
.I RTFGetInputName()
and
.I RTFGetOutputName() .
If this is to be done, the
driver is responsible for calling
.I RTFSetInputName()
and
.I RTFSetOutputName()
beforehand.
(The driver must do this after calling
.I RTFInit()
but before calling
.I WriterBeginFile() .)
.sp .5v
The driver must ensure that file pathnames are
C strings in a form that can be
.I fopen() -ed
as is.
Other than that,
the writer cannot assume much about the contents of pathnames.
For instance, pathname components are separated by ``/'' under UNIX, but
by ``:'' on a Macintosh.
.sp .5v
The input and/or output names may be
.Cw NULL ,
which indicates that no file name
is associated with the given stream.
(This may happen, for example, if the
translator is reading from or writing to a pipe under UNIX).
One reason for wanting to know the name of the input file is if the writer
wants to compute statistics on the input and report them in reference to
the input name (for example, as
.I rtfwc
does to show the user which input
file the output statistics are for).
One reason the writer might want to know the output file name is when the
writer writes multiple output files. In this case, it is convenient to use
the primary output file name as a basis for deriving related file names.
.sp .5v
The writer is free to abort if an output file name is required.
.Li
.I WriterBeginFile()
installs token class callbacks into the reader for writer functions that
should be invoked when instances of particular token classes occur.
.I RTFInit()
resets all the callbacks, so they
must be installed for each file processed.
This is one reason
.I WriterBeginFile()
must be called after
.I RTFInit()
and not before.
Another reason is that
the writer
might also read a few tokens from the input stream to make sure the
input file looks like a valid RTF file (e.g., beginning with
``{'' and ``\ertf'').
This can only be done after
.I RTFInit()
has been called.
.Li
.I WriterBeginFile()
determines the status value to return to the driver.
The value
should be non-zero if
.I WriterBeginFile()
is able to
set up for translating a new file, or zero
if some error occurs that
prevents translation of the input file.
.sp .5v
If
.I WriterBeginFile()
fails, it should generate its own error
message, not leave that for the driver.
If a failure occurs, the driver
decides whether to abort or to continue with any further files that may be
queued for translation.
.Le
.\"
.Bh "Interface Guidelines Summary"
.\"
.Ls B
.Li
Install OS-specific callbacks when the translator starts up.
.Li
To generate diagnostic message that aren't written to the primary output
file, call
.I RTFMsg() .
Don't write to
.I stderr ,
since there might not be any open file associated with it.
.Li
To generate an error message and terminate the process in error, call
.I RTFPanic() .
Don't write to
.I stderr ,
and don't call
.I exit() .
.Li
To open a library file, pass the file's basename and open mode to
.I RTFOpenLibFile() .
.Li
The driver is responsible for opening the input file on
.I stdin .
.Li
Either the driver or the writer may open the output file.
If the driver opens it, it should do so on
.I stdout .
.Li
If the writer needs to know the input or output file names,
the driver is responsible to set them with
.I RTFSetInputName()
and
.i RTFSetOutputName() .
This must be done after calling
.I RTFInit()
and before calling the writer's begin-file function.
The writer may then obtain the names with
.I RTFGetInputName()
and
.i RTFGetOutputName() .
If the writer wants to modify the strings returned by these functions,
it should make copies of them.
.Le
.\"
.Bh "Handling User Preferences"
.\"
.LP
This section is, uh, not very good yet.
