.\"
.\" $(TROFF) $(MSMACROS) file
.\"
.\" revision date - change whenever this file is edited
.ds RD 5 April 1994
.nr PO 1.25i	\" page offset 1.25 inches
.nr PD .5v	\" inter-paragraph distance
.\"
.EH 'RTF Character Mapping'- % -''
.OH ''- % -'RTF Character Mapping'
.OF 'Revision date:\0\0\*(RD''Printed:\0\0\n(dy \*(MO 19\n(yr'
.EF 'Revision date:\0\0\*(RD''Printed:\0\0\n(dy \*(MO 19\n(yr'
.\"
.\" subscript strings
.ds < \s-2\v'.4m'
.ds > \v'-.4m'\s+2
.\"
.\" I - italic font (taken from -ms and changed)
.de I
.nr PQ \\n(.f
.if t \&\\$3\\f2\\$1\\fP\&\\$2
.if n .if \\n(.$=1 \&\\$1
.if n .if \\n(.$>1 \&\\$1\c
.if n .if \\n(.$>1 \&\\$2
..
.TL
RTF Tools
.sp .2v
Character Mapping
.AU
Paul DuBois
dubois@primate.wisc.edu
.AI
Wisconsin Regional Primate Research Center
Revision date:\0\0\*(RD
.\"
.NH
Introduction
.LP
Text characters in an RTF file may be specified as literal characters
or using \e\'\fIxx\fP notation, where
.I xx
is the hex value of the character.
RTF files also
contain a control word that specifies the character set that's used within
the document and governs the interpretation of character values.
The charset control words are:
.LP
.DS
.ta 1i
\eansi	ANSI (default)
\emac	Apple Macintosh
\epc	IBM PC
\epca	IBM PC page 850, used by IBM Personal System/2
.DE
.LP
Although the four charsets don't appear to differ
for characters in the ASCII range (below 128), they differ
considerably above the ASCII range (128\-255).
For example, the ANSI, Macintosh, and PC charsets represent
the degree sign (``\(de'') as \e\'b0, \e\'a1, and \e\'f8.
Furthermore, even for a given charset, character values in the Symbol
font represent different characters than they do generally.
For example ``a'' in Symbol font is ``\(*a''.
.LP
The RTF Tools distribution uses a translation model that divides translation
effort between reader code that tokenizes the RTF input stream
and writer code that decides what to write based on the current token.
The reader is common to all translators; writer code differs on a
per-translator basis.
In order to free writer code from
charset dependencies, the reader performs input character mapping
onto a charset-independent set of character codes.
This makes it unnecessary to duplicate charset-dependent decisions in
each writer.
.NH
Character Translation
.LP
Input RTF characters are mapped to output sequences like this:
.IP \(bu
When a text character (i.e., a token in the
.I rtfText
class) is read, the
reader maps it onto a ``standard''
character code, using a translation table selected by taking into account
the charset used by the input file and whether or not the font is Symbol.
.I rtfMajor
and
.I rtfMinor
are set to the input character value and the standard character code,
respectively.
If there is no entry for the RTF character in the translation table,
.I rtfMinor
is set to
.I rtfSC_nothing
(0).
.IP \(bu
At output time, the writer maps the standard character code to an appropriate
output sequence.
Typically the writer does this by consulting an output map that specifies
the appropriate output sequence for each standard character code.
.LP
Using this
method, the writer has access to the original input character
if it
needs it for some reason
.I rtfMajor ), (
but also can work with a
charset-independent representaton of the character
.I rtfMinor ). (
.NH 2
Standard Character Representation
.LP
Charset maps and output maps are represented as simple text files that are
read when a translator runs.
In map files, the standard characters are represented by names
patterned after those used in Adobe Font Metric (AFM) files.
For example, ``space'', ``quotedblleft'', and ``bullet'' represent the
space, left double quote, and bullet characters.
When map files are read in, the standard names are
converted to integers in the range 0..\fIrtfSC_MaxChar\fP\-1, where
.I rtfSC_MaxChar
is the number of standard characters recognized.
.LP
The set of standard characters contains characters
represented by the charsets.
It also contains characters represented by certain RTF ``special-character''
control words, such as ``nobrkspace'' for the unbreakable-space control
word ``\e~''.
.LP
By keeping charset and output sequence maps in files instead of
compiling them in, translations can be changed or fixed
by editing text files rather than by recompiling programs.
.NH 2
Map Locations
.LP
The charset map files are found within the distribution in the
.I LIBSRC
directory and are installed in
.I LIBDIR .
Individual translators provide output maps in their own source directory
and install the maps in
.I LIBDIR .
.LP
The map reading functions
.I RTFReadCharsetMap()
and
.I RTFReadOutputMap()
look for charset and output
map files in the current directory before looking in
.I LIBDIR .
This allows users to copy maps from
.I LIBDIR
into their current directory,
modify the copies, and have the translators use the modified copies instead.
.NH 2
Charset Map Format
.LP
General and symbol charset maps are stored in the text files
.I ansi-gen ,
.I ansi-sym ,
.I mac-gen ,
.I mac-sym ,
.I pc-gen ,
.I pc-sym ,
.I pca-gen ,
and
.I pca-sym .
Each line of a charset file associates an RTF character value (field 2)
with the standard character name to which
the RTF character corresponds (field 1).
Here's a sample from
.I ansi-gen :
.LP
.DS
.ta 1.5i
parenleft	(
parenright	)
space		"\0"
quotedbl	\'"\'
quoteright	"\'"
quoteleft	"\`"
a		a
b		b
c		c
bullet		0x95
emdash		0x96
endash		0x97
.DE
.LP
Character values may be given as a single character (in which case the ASCII
value is used), or as a hex number
.I yy . 0x
Single or double quotes may be used to quote values containing whitespace
or quotes (e.g., use single quotes to quote a double-quote).
.LP
Lines with a ``#'' in column one are taken as comments.
Comments and blank lines are ignored.
.NH 2
Output Sequence Map Format
.LP
Writers can encode the map associating standard character codes and
output sequences any way they want.
However, a map can be stored in a text file and read easily with
.I RTFReadOutputMap()
if the map file uses a format such that:
(i)
field 1 is the standard character name, just as in the charset maps;
(ii)
field 2 is the output sequence to produce for the character named in field
1;
(iii)
comments are specified as in charset maps.
.LP
Here's part of the
.I groff-map
file used by
.I rtf2troff
to produce output for
.I groff :
.LP
.DS
.ta 1.5i
a		a
b		b
c		c
quotedblleft	"\`\`"
quotedblright	"\'\'"
bullet		\e(bu
emdash		\e(em
AE		\e(AE
ae		\e(ae
.DE
.LP
To read an output file, declare an array of character pointers to hold
the output sequences and call
.I RTFReadOutputMap() :
.LP
.DS
char	*outMap[rtfSC_MaxChar];

if (RTFReadOutputMap (outputMapName, outMap, 1) == 0)
	RTFPanic ("Cannot read output map %sn", outputMapName);
.DE
.LP
Like
.I RTFReadCharSet() ,
.I RTFReadOutputMap()
looks first in the current directory and then
.I LIBDIR
for the file (unless the file is specified as an absolute pathname).
.LP
The last argument to
.I RTFReadOutputMap()
is 1 or 0, depending on whether or not you want the map to be initialized
before the file is read.
(It is sometimes useful to construct a map from several sources, in which
case you initialize the map only for the first file read, not the
second and subsequent ones.
See
.I rtf2troff
for an example.)
.LP
To convert a standard character code to an output sequence, use the code
as in index into the output map array:
.LP
.DS
seq = outMap[rtfMinor];
.DE
.LP
Output mapping varies from simple to relatively complex.
.I rtf2text
uses a single output map,
.I text-map.
For
.I rtf2troff ,
it's not so simple.
.I rtf2troff
selects an output map that's specific to the version of
.I troff
you want output for
.I xroff-map , (
.I groff-map ,
etc.).
This is because special characters are typically represented in
.I troff
by escape sequences (such as ``\e(de'' for the degree symbol),
but different versions of
.I troff
support differing sets of escape sequences.
If
.I rtf2troff
can't find a map for your version of
.I troff ,
it reads a ``generic''
.I troff
map.
.LP
If an
.I \-m
argument is given on the command line,
.I rtf2troff
also looks for an output map corresponding to the named macro package
and combines it with the
.I troff -version
map.
This is because macro packages sometimes provide escape sequences
of their own to support additional special characters.
.NH
Dealing with Missing Map Entries
.LP
.B "Problem 1:"
The charset maps provided in the distribution are fairly complete.
However, it is still possible that a map will have no entry for a
given input character.
In this case, the reader sets
.I rtfMinor
to
.I rtfSC_nothing .
This is a legal character code, and you can treat it various ways.
One approach is to put an entry for the name ``nothing'' in your output map,
e.g.,
.LP
.DS
nothing	"[[CHARACTER DROPPED]]"
.DE
.LP
This way, when the writer translates characters to output sequences, it need
make no special effort for characters that are missing from the charset
map, and an obvious marker appears in the output.
However, this approach doesn't provide any clue about what the character was.
.LP
Alternatively, you can try to show some
representation of missing characters in the output:
.LP
.DS
if (rtfMinor != rtfSC_nothing)
	/* print normal output sequence here */
else if (rtfMajor < 128)	/* in ASCII range */
	printf ("[[%c]]", rtfMajor);
else				/* non-ASCII, print hex value */
	printf ("[[\e\e\'%02x]]", rtfMajor)
.DE
.LP
.B "Problem 2:"
If there is no entry for a standard character code in the
output map, the
output sequence will be NULL.
One way to deal with this is to print the standard character name:
.LP
.DS
char *seq;

seq = outMap[rtfMinor];
if (seq != (char *) NULL)
	fputs (seq, stdout);
else
	printf ("[[%s]]", RTFStdCharName (rtfMinor));
.DE
.NH
Additional Charsets
.LP
The mapping mechanism described here handles the four existing charsets
uniformly.
If/when the RTF specification is modified to allow
additional charsets, it should be
easy to provide support for them with no change to writer code:
.IP \(bu
Add a #define for the charset control word to
.I rtf.h .
.IP \(bu
Add an entry for the control word to the
.I rtfKey [\^]
array in
.I reader.c .
.IP \(bu
Create and install general and symbol charset map files.
.LP
With those changes made, translators can simply be recompiled and
should automatically work for RTF files using the new charset.
