/*
 * MyStandardFile.c
 *
 * Implements function MyStandardardGetFile() and MyStandardPutFile()
 * that take the same arguments as StandardGetFile() and StandardPutFile()
 * but that cause the TransSkel standard dialog event filter to be
 * used so that updates/activates for underlying windows are processed
 * properly.  These are implemented by mapping the arguments onto calls
 * to CustomGetFile() and CustomPutFile().
 */

# include	"TransSkel.h"
# include	"MyStandardFile.h"

/*
 * FileFilterWrapper() translates the file filter used in the call
 * to CustomGetFile() into the format used by StandardGetFile().
 * If the caller supplied no filter, all files are accepted for
 * display.
 */

static FileFilterProcPtr	wrapperFileFilter;

static pascal Boolean
FileFilterWrapper (ParmBlkPtr PB, void *data)
{
	if (wrapperFileFilter != nil)
		return ((*wrapperFileFilter) (PB));	/* ignore data */
	return (false);
}


pascal void
MyStandardGetFile (FileFilterProcPtr fileFilter, short numTypes,
			SFTypeList typeList, StandardFileReply *reply)
{
Point	where;

	/*
	 * Set up.  Save pointer to fileFilter in global variable so
	 * FileFilterWrapper() use it.  Set where to (-1, -1) so dialog
	 * will be centered on screen.
	 */
	wrapperFileFilter = fileFilter;	/* save pointer to filter in global */
	SetPt (&where, -1, -1);			/* center dialog on screen */

	/*
	 * Call file dialog, mapping caller's arguments into CustomGetFile()
	 * and supplying dialog filter.
	 */
	CustomGetFile (FileFilterWrapper,				/* fileFilter wrapper */
					numTypes,
					typeList,
					reply,
					0,								/* use default template */
					where,							/* center on screen */
					nil,							/* no dialog hook */
					SkelDlogFilterYD (nil, false),	/* add filter here */
					nil,							/* no activeList */
					nil,							/* no activateProc */
					nil);							/* no data */

	/* remove dialog filter */

	SkelRmveDlogFilter ();
}
