/*
 * rtf-unix.c -- UNIX rtf2troff driver.
 *
 * Syntax: rtf2troff options file
 *
 * Options:
 *	-e		echo token information
 *	-h		disable header/footer macro/trap output (default)
 *	+h		enable header/footer macro/trap output
 *	-me		write -me specific output
 *	-mm		write -mm specific output
 *	-ms		write -ms specific output
 *	-s		disable strikethrough
 *	+s		enable strikethrough (default)
 *	-t troff	specify troff version
 *	-u		disable underlining
 *	+u		enable underlining (default)
 */

# include	<stdio.h>
# include	<sys/types.h>
# include	<ctype.h>

# include	"rtf.h"
# include	"rtf-unix.h"
# include	"rtf2troff.h"


static char	*usage = "Usage: rtf2troff [-e] [-h|+h] [-man|-me|-mm|-ms] [-s|+s] [-t troff] [-u|+u] file";


int
main (argc, argv)
int	argc;
char	*argv[];
{
	/* install OS-specific callbacks */
	UnixSetProgPath (argv[0]);
	RTFSetOpenLibFileProc (UnixOpenLibFile);

	/*
	 * Process arguments.  The -t and -man|-me|-mm|-ms arguments
	 * must be remembered and applied *in that order* after
	 * argument processing, or char maps may not be selected
	 * properly.
	 */
	--argc;
	++argv;
	while (argc > 0 && (**argv == '-' || **argv == '+'))
	{
		if (strcmp ("-e", *argv) == 0)
			SetEcho (1);
		else if (strcmp ("-h", *argv) == 0)
			doHeaders = 0;
		else if (strcmp ("+h", *argv) == 0)
			doHeaders = 1;
		else if (strcmp ("-s", *argv) == 0)
			allowStrikeThru = 0;
		else if (strcmp ("+s", *argv) == 0)
			allowStrikeThru = 1;
		else if (strcmp ("-u", *argv) == 0)
			allowUnderline = 0;
		else if (strcmp ("+u", *argv) == 0)
			allowUnderline = 1;
		else if (strcmp ("-t", *argv) == 0)
		{
			if (argc < 2)
			{
				fprintf (stderr, "%s\n", usage);
				exit (1);
			}
			--argc;
			++argv;
			SetTroffVersion (*argv);
		}
		else if (strcmp ("-man", *argv) == 0)
			SetMacroVersion (*argv);
		else if (strcmp ("-me", *argv) == 0)
			SetMacroVersion (*argv);
		else if (strcmp ("-mm", *argv) == 0)
			SetMacroVersion (*argv);
		else if (strcmp ("-ms", *argv) == 0)
			SetMacroVersion (*argv);
		else
		{
			RTFPanic ("Unknown option: %s\n%s", *argv, usage);
		}
		--argc;
		++argv;
	}


	/* not clever; only allows stdin or one named file to be read */

	if (argc > 0)
	{
		if (freopen (argv[0], "r", stdin) == (FILE *) NULL)
			RTFPanic ("Can't open \"%s\"", argv[0]);
	}

	WriterInit ();

	RTFInit ();
	if (BeginFile ())
	{
		RTFRead ();		/* read everything else */
		EndFile ();
	}

	exit (0);
}
