/*
 * diag-writer.c - read rtf input, write diagnostic stuff.  Useful for
 * testing reader to see if it's finding and classifying
 * tokens properly, reading destinations, reprocessing
 * styles, etc.
 *
 * Driver should open output file on stdout.
 *
 * 03 Feb 91	Paul DuBois	dubois@primate.wisc.edu
 *
 * 03 Feb 91 V1.0
 * - Created.
 * 27 Feb 91 V1.01
 * - Updated for distribution 1.05.
 * 13 Jul 93 V1.02
 * - Updated to compile under THINK C 6.0.
 * 12 Mar 94
 * - Updated for 1.10 distribution.
 */

# include	<stdio.h>

# include	"rtf.h"
# include	"rtfdiag.h"


static void	TokenEcho ();
static void	PrintTables ();


static int	echo = 0;


int
BeginFile ()
{
	if (echo)
		RTFSetReadHook (TokenEcho);
	else
		RTFSetReadHook (NULL);
	return (1);
}


void
EndFile ()
{
	PrintTables ();
}


void
SetEcho (newEcho)
int	newEcho;
{
	echo = newEcho;
}


static void
TokenEcho ()
{
	printf ("%8d %7d %7d %9d \"%s\"\n",
			rtfClass, rtfMajor, rtfMinor, rtfParam, rtfTextBuf);
}


static void
PrintTables ()
{
RTFColor	*cp;
RTFFont		*fp;
RTFStyle	*sp;
RTFStyleElt	*sep;
int	count;

	printf ("Font table:\n");
	count = 0;
	for (fp = RTFGetFont (-1); fp != NULL; fp = fp->rtfNextFont)
	{
		++count;
		printf ("%2d: %s %d\n", fp->rtfFNum, fp->rtfFName,
			fp->rtfFFamily);
	}
	printf ("Font table entries: %d\n\n", count);

	printf ("Color table:\n        red   green    blue\n");
	count = 0;
	for (cp = RTFGetColor (-1); cp != NULL; cp = cp->rtfNextColor)
	{
		++count;
		printf ("%2d: %7d %7d %7d\n", cp->rtfCNum, cp->rtfCRed,
			cp->rtfCGreen, cp->rtfCBlue);
	}
	printf ("Color table entries: %d\n\n", count);

	printf ("Stylesheet:\n");
	count = 0;
	for (sp = RTFGetStyle (-1); sp != NULL; sp = sp->rtfNextStyle)
	{
		++count;
		printf ("%2d: %s %d %d\n", sp->rtfSNum, sp->rtfSName,
				sp->rtfSBasedOn, sp->rtfSNextPar);
		for (sep = sp->rtfSSEList; sep != NULL; sep = sep->rtfNextSE)
			printf (" %s\n", sep->rtfSEText);
	}
	printf ("Style table entries: %d\n\n", count);
}
